% PURPOSE: Matsuoka,H. (2022), Debt Intolerance: Threshold Level and Composition
% Oxford Bulletin of Economics and Statistics, vol. 84(4), pp. 894-932
%
% -------------------------------------------------------
% Usage:   Estimate a Panel Smooth Threshold Panel
% -------------------------------------------------------

function [res]=GPSTR(Y,Q,X,Zx,z1,z2,z3,z4,z5,N0,condini_user)

warning off;

affi=1;

global  T N K Xreg beta residual y_cent z_cent x_cent w_cent g r condini_last params ncase NT sample Qadj Xadj sampleAdj qs_truncated

%Y X W Q
data=[Y,Q,X,Zx];
OPTIONS =  optimset('MaxIter',20000,'Display','on','MaxFunEvals',20000);

%------------------
%--- Parameters ---
%------------------

%m=m0;                                              % Assumed number of location Paramter
                                  
    grid_gam1=21;  % Number of grid points for gamma 
    grid_gam2=21;
    grid_c=30;                                     % Number of grid points for each c (must be superior to m)



alpha=0.05;                                     % Level of Tests (initial tests in the sequence)

tau=0.5;                                        % Factor of Reduction of fisrt order level in sequential procedure

disp(' ')
disp(' PLEASE CITE: ')
disp(' ')
disp(' Matsuoka,H. (2022), Debt Intolerance: Threshold Level and Composition')
disp(' Oxford Bulletin of Economics and Statistics, vol. 84(4), pp. 894-932')
disp(' ')
disp(' Thank you')
disp(' ')

%------------
%--- Data ---
%------------

Y=data(:,1);                                    % Endogeneous Variable

Q=data(:,2);                                    % Threshold Variable (Identical for all threshold variables)

X=data(:,3);                                   % Matrix of Explicative Variables

N=N0;                                           % Number of Individual 

T=length(Y)/N;                                  % Time Dimension
 
NT=N*T;                                         % Total number of observations

K=size(X,2);                                    % Number of Explicative Variables
L=size(Zx,2);
res.L=L;
%-------------------------
%---- Balanced Sample ----
%-------------------------

balanced=1;                                                             % balanced=1 if the sample is balanced 

if sum(sum(isnan(data)))>0                                              % Not Balanced Sample  

    disp(' '),disp(' WARNING: The panel is not balanced'),disp(' ')    
    
    balanced=0;                                                         % balanced=0 if the sample is not balanced 
       
end

missing=sum(isnan(data)')';                                             % Location of Missing Values                         

sample=zeros(NT,3);                                                     % Sample of Complete Data        

sample(:,1)=ones(length(Y),1);sample(missing>0)=0;                      % Sample of Complete Data        
        
balanced_indi=repmat((1:N),T,1);sample(:,2)=balanced_indi(:);           % Index of Individuals
    
sample(:,3)=repmat((1:T),1,N)';                                         % Index of Time
   
Qadj=Q(sample(:,1)==1);                                                 % Values of Qit on the Adjuted Sample

Xadj=X(sample(:,1)==1,:);                                               % Values of Xit on the Adjuted Sample
Zxadj=Zx(sample(:,1)==1,:); 
sampleAdj=sample(sample(:,1)==1,:);                                     % Adjusted Sample

NT=sum(sample(:,1));                                                    % Total number of observations


% Vector of Centered Data
y_cent=centered(Y,sample);                                   
z1_cent=centered(z1,sample);
z2_cent=centered(z2,sample);
z3_cent=centered(z3,sample);
z4_cent=centered(z4,sample);
z5_cent=centered(z5,sample);
z_cent=[z1_cent z2_cent z3_cent z4_cent z5_cent];

% Centered Explicative Variables X(i,t)

for indic_k=1:K                                             % Loop on the number of explicative variable
      
    x_cent(:,indic_k)=centered(X(:,indic_k),sample);         % Centered Vector of X
    
end


if nargin==5
    
    disp('WARNING: Initial Condition Provided by User')
    
else                                               % Begin of the condition on initial condition provided by user

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                      % 
% Testing Linearity against GPSTR model %
%                                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if affi==1, disp(' ')
    
    disp(' ***********************')
    disp(' *** LINEARITY Tests ***')
    disp(' ***********************')
    
    disp(' ')
    
end  

% Computation of SSR0 (Within Estimate, Linear Model)
LinearRegrssor=[z_cent x_cent];
within=LinearRegrssor\y_cent; % Within Estimate Linear Model                                      
SSR0=sum((y_cent-LinearRegrssor*within).^2); 

% Residual Sum of Square Linear Model
TSS=sum((y_cent-mean(y_cent)).^2);

% Computation of SSR1 (Taylor expansion around gam=0)
Regressor=x_cent;                                             % Matrix of Regressors

    XQj=X.*repmat(Q,1,K);                                  % Transformed Variable X*Q    
    
    for indic_k=1:K                                           % Loop on the number of explicative variable
        
        XQj_cent(:,indic_k)=centered(XQj(:,indic_k),sample);  % Centered Vector of X*Q
        
    end                                                       % End on the loop on K  
    
    Regressor=[Regressor XQj_cent];                           % Matrix of Regressors
    

Taylarregressor=[z_cent Regressor];   
beta_taylor=Taylarregressor\y_cent;                                 % Estimate Linear Model approximation
SSR1=sum((y_cent-Taylarregressor*beta_taylor).^2);                   % Residual Sum of Square 
      

% Statistics of Linearity Tests

LM=NT*(SSR0-SSR1)/SSR0;                                        % Wald Statistic Test (LM Statistic, GTVD, page 4)

LM_linearity(1,1)=LM;                                          % Wald Statistic Test (LM Statistic, GTVD, page 4)

LM_linearity(1,2:4)=chi2inv([0.90 0.95 0.99],K);            % Critical Values at 10%, 5% and 1 (Version 7.00)%

LM_linearity(1,5)=1-chi2cdf(LM,K);                          % P-value on LM statistic (Version 7.00)

degree=length(y_cent)-(size(Taylarregressor,2))-N;             % Degree of Freedom 
F=((SSR0-SSR1)/(K))/(SSR1/degree);                           % Fisher Statistic Test (LMF Statistic, GTVD, page 4)

F_linearity(1,1)=F;                                            % Fisher Statistic Test (LMF Statistic, GTVD, page 4)

F_linearity(1,2:4)=finv([0.90 0.95 0.99],K,degree);      % Critical Values at 10%, 5% and 1% (Verrsion 7.00)

F_linearity(1,5)=1-fcdf(F,K,degree);                     % Pvalue for the F-statistic (Vesrion 7.00)

LRT=NT*(log(SSR0)-log(SSR1));                                  % LRT Statistic Test (LMF Statistic, GTVD, page 4)

LRT_linearity(1,1)=LRT;                                        % LRT Statistic Test (LMF Statistic, GTVD, page 4)

LRT_linearity(1,2:4)=chi2inv([0.90 0.95 0.99],K);      % Critical Values at 10%, 5% and 1% version 7.00

LRT_linearity(1,5)=1-chi2cdf(LRT,K);                     % Pvalue for the F-statistic version 7.00

if affi==1
    
    disp(' H0: Linear Model H1: GPSTR model with at least one Threshold Variable (r=1) '),disp(' ')

    disp(sprintf ('     Wald Tests (LM):         W = %3.3f    pvalue = %1.3f',LM_linearity(1), LM_linearity(end))),disp(' ')
    
    disp(sprintf ('     Fisher Tests (LMF):      F = %3.3f    pvalue = %1.3f',F_linearity(1), F_linearity(end))),disp(' ')
    
    disp(sprintf ('     LRT Tests (LRT):         LRT = %3.3f    pvalue = %1.3f',LRT_linearity(1), LRT_linearity(end))),disp(' ')
    
end

res.LR=LRT_linearity(1);

res.LR_pvalue=LRT_linearity(end);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                              %
% Determination of The Number of Threshold r   %
%                                              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if affi==1, disp(' '),disp(' '),
    
    disp(' **************************************************************************')
    disp(' *** TESTING THE NUMBER OF REGIMES: TESTS OF NO REMAINING NON-LINEARITY ***')
    disp(' **************************************************************************')
        
end  


pvalue=0;critical=alpha;r=0;

LM=zeros(1,5);                                                        % Stats LM, Critcial Values and pvalues

F=zeros(1,5);                                                         % Stats LMF, Critcial Values and pvalues

LRT=zeros(1,5);                                                       % Stats LRT, Critcial Values and pvalues
    
while (pvalue<critical)

    r=r+1;                                                               % Increment in r

    %--- Initial Conditions ---

    if r>1; condini_last=parameter; end                                  % Last Initial Condition 
          
                                   % Grid Search for Initial Values
   condini=grid_search(grid_gam1,grid_gam2,grid_c);   
    %--- Estimation ---

    [parameter,rss,EXITFLAG,OUTPUT] = fminsearch('NLS',condini,OPTIONS); % NLS                              
    
    disp(sprintf('  Estimation of the Model with r = %0.0f : Convergence = %0.0f   RSS = %0.3f ',r,EXITFLAG,rss))   
    
        
    %--- Test of no Remaining nonlinearity ---
        
    SSR0=rss;                                                     % Residual Sum of Square of Model with r Regimes (r>=1)
    
    Regressor=[Xreg];                                             % Xreg is the matrix of Regressor for model with r Threshold
    

        XQj=X.*repmat(Q,1,K);                                      % Transformed Variable X*Q    
           
        for indic_k=1:K                                           % Loop on the number of explicative variable
        
            XQj_cent(:,indic_k)=centered(XQj(:,indic_k),sample);  % Centered Vector of X*Q            
            
        end                                                       % End on the loop on K  
    
        Regressor=[Regressor XQj_cent];                           % Matrix of Regressors
    
    
   coef_H1=Regressor\y_cent;                                      % Model under H1

   SSR1=sum((y_cent-Regressor*coef_H1).^2);                       % Conditionnal RSS

   disp(sprintf('  RSS under H1 = %0.3f ',SSR1))   
    
   LM(r,1)=NT*(SSR0-SSR1)/SSR0;                                    % Wald Statistic Test (LM Statistic, GTVD, page 4)
    
   LM(r,2:5)=chi2inv([0.90 0.95 0.99 1-critical],K);            % Critical Values at 10%, 5% and 1%
 
   LM(r,6)=1-chi2cdf(LM(r,1),K);                                %  P-value on LM statistic 
   
   degree=NT-N-K*(r+1)-K;                                        % Degree of Freedom (not the same as in the article)
   
   F(r,1)=((SSR0-SSR1)/(K))/(SSR1/(degree));                     % Fisher Statistic Test (LMF Statistic, GTVD, page 4)

   F(r,2:5)=finv([0.90 0.95 0.99 1-critical],K,(degree));    % Critical Values at 10%, 5% and 1%

   F(r,6)=1-fcdf(F(r,1),K,(degree));                         % Pvalue on LMF statistic
    
   LRT(r,1)=NT*(log(SSR0)-log(SSR1));                               % Wald Statistic Test (LM Statistic, GTVD, page 4)
    
   LRT(r,2:5)=chi2inv([0.90 0.95 0.99 1-critical],K);            % Critical Values at 10%, 5% and 1%
 
   LRT(r,6)=1-chi2cdf(LRT(r,1),K);                                %  P-value on LM statistic
   
   
    %--- Adjsutement of the Level ---
    
    critical=critical*tau;                                       % Reduction of the level of test: factor tau (cf. Teravirsta and all., 2004)
    
    pvalue=F(r,end);                                             % We use only here the Fversion of the test (cf. page 12)

end  

if affi==1 
    
    disp(' ')

    for j=1:r
    
        disp(sprintf(' H0: GPSTR with r = %1.0f  against  H1: PSTR with at least r = %1.0f ',j,j+1)),disp(' ')

        disp(sprintf ('     Wald Tests (LM):         W = %3.3f    pvalue = %1.3f',LM(j,1), LM(j,end))),disp(' ')
    
        disp(sprintf ('     Fisher Tests (LMFG):      F = %3.3f    pvalue = %1.3f',F(j,1), F(j,end))),disp(' ')
    
        disp(sprintf ('     LRT Tests (LRT):         LRT = %3.3f    pvalue = %1.3f',LRT(j,1), LRT(j,end))),disp(' ')
    
    end
    
    disp(' ')
    
    
end


res.LR1=LRT(:,1);

res.LR1_pvalue=LRT(:,end);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                              %
% Estimation by Non Linear OLS %
%                              %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

end % END sur la condition sur consini_user. 


% Estimation NLS

if nargin==5                                % Case of Initial Condition Provided by User
    
    condini=condini_user;
    
gam_def1=(0.2:0.8/(floor(grid_gam1/4)-1):1)';     % Grid on gamma values
gam_def1=(0.2:0.8/(floor(grid_gam1/4)-1):1)';     % Grid on gamma values
gam_def2=[gam_def1 ; (2:(100-2)/(grid_gam2-floor(grid_gam2/4)-1):100)'];
gam_def2=[gam_def2 ; (2:(100-2)/(grid_gam2-floor(grid_gam2/4)-1):100)'];
    qs=sort(Qadj);                                   % Sorted Values of Qit           

    indic=(floor(0.15*length(qs)):1:floor(0.85*length(qs)))';

    qs_truncated=qs(indic);                         % Truncated sample: 30% of observations are withdraw
    
else

    condini=parameter;
    
end;


if affi==1

  disp(' '),disp(' '),
  
  disp(' **************************************')
  disp(' *** FINAL ESTIMATION OF PSTR MODEL ***')
  disp(' **************************************')
  
  disp(' ')
  
  disp(sprintf('   Final Estimation of the Model with r = %0.0f by NLS ***',r))  , disp(' ')
   
end

disp('   Initial Conditions on (gamma,c) :')

disp(condini')

[parameter,rss,EXITFLAG,OUTPUT] = fminsearch('NLS',condini,OPTIONS);

%---- Check up of the Estimates : verify if the location parameters are inside the trimming values of q(it) ---


% Initialisation    
m1=zeros((r+1)*(K)+L,(r+1)*(K)+L);m2=m1;        
for indic=1:NT                                                      % With Hetero (see. Hansen 1999)
    
    m1= m1 + Xreg(indic,:)'*Xreg(indic,:);           
           
    m2= m2 + Xreg(indic,:)'*Xreg(indic,:)*residual(indic)^2;
    
end
vcov2=inv(m1)*m2*inv(m1);                                           % Variance Covaraince Corrected for Heteroskedasticity 

% Residuals 
resid=full(sparse(sampleAdj(:,3),sampleAdj(:,2),residual,T,N));
resid=resid(:);resid(resid==0)=NaN;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Driscoll-Kray standard errors%                             
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%            stdDK         (K*L)x1 vector, Driscoll-Kraay standard errors
%            stdW          (K*L)x1 vector, White's standard errors
%            CovDK         (K*L)x(K*L) matrix, Driscoll-Kraay covariance matrix
%            CovW          covariance matrix, White's
%            CovDKj        covariance matrix, DK with lags
%            stdDKj        standard errors, DK with lags


x_cent_DK=reshape(x_cent,T,N);
z1_cent_DK=reshape(z1_cent,T,N); 
z2_cent_DK=reshape(z2_cent,T,N);  
z3_cent_DK=reshape(z3_cent,T,N);  
z4_cent_DK=reshape(z4_cent,T,N); 
z5_cent_DK=reshape(z5_cent,T,N); 
XQj_cent_DK=reshape(XQj_cent,T,N); 

x_DK=cat(3,x_cent_DK,XQj_cent_DK,z1_cent_DK,z2_cent_DK,z3_cent_DK,z4_cent_DK,z5_cent_DK);

xx = 0;                              %Sum[x(t)*x(t)',t=1:T]
KL=K+L+r;
DKlag=1;
ScaleByNtQ=1;
yhatQ = 1;

for t = 1:T;                                     %loop over time
  x_t   = reshape(double(x_DK(t,:,:)),N,r+K+L);  %NxL, better than squeeze (cf 2-d arrays)
  if ScaleByNtQ == 1;
    Nb(t) = size(x_t,1);
  else;
    Nb(t) = N;
  end;
    xx  = xx + x_t'*x_t/Nb(t);
end;

Tb  = sum(Nb>0);                    %number of effective time periods
xx  = xx/Tb;
if yhatQ == 1;
  yhat = NaN(T,N);
else;
  yhat = [];
end;
omega0DK = zeros(KL,KL);               %DK, lag 0
omega0W  = omega0DK;                   %White's
omegajDK = zeros([KL,KL,DKlag]);           %DK, lags 1 to m
h_tLag   = zeros(DKlag,KL);                %lag1;lag2;...,lagm
resi=reshape(residual,T,N);
resi_lag1=resi((1:size(resi,1)-1),:);
resi_t=resi((2:size(resi,1)),:);
SSR_lag1=sum(sum(resi_lag1.*resi_t));
SSR=sum(sum(resi.*resi));




for t = 1:T;                           %loop over time
  
  x_t    = reshape(double(x_DK(t,:,:)),N,K+L+r);
   r_t    = resi(t,:)';
   rx_t   = excise([r_t,x_t]);
   
   if ~isempty(rx_t);                   %don't accumulate [] to omega0DK
     r_t    = rx_t(:,1);
     x_t    = rx_t(:,2:end);
    hi_t   = x_t.*repmat(r_t,1,KL);    %moment condition for (i,t)

    h_t    = sum(hi_t,1)/Nb(t);
    omega0DK = omega0DK + h_t'*h_t;
    omega0W  = omega0W + hi_t'*hi_t/Nb(t)^2;
    for j = 1:DKlag;
      omegajDK(:,:,j) = omegajDK(:,:,j) + h_t'*h_tLag(j,:);    %h(t)*h(t-j)'
    end;
    h_tLag = [h_t;h_tLag(1:end-1,:)];  %update only if ~isempty(rx_t), effectively disregarding t if no data
  end;

end;

Shat  = omega0DK/Tb^2;                   %estimate of S, DK
Shatw = omega0W/Tb^2;                    %estimate of S, White's
Shatj = omega0DK/Tb^2;
for j = 1:DKlag;
  Shatj = Shatj + (1-j/(DKlag+1))*(omegajDK(:,:,j)+omegajDK(:,:,j)')/Tb^2 ;
end;

zx_1  = inv(xx);
CovDK = zx_1 * Shat * zx_1';                      %covariance matrix, DK
stdDK = sqrt( diag(CovDK) );                      %standard errors, DK
CovW  = zx_1 * Shatw * zx_1';                     %covariance matrix, White's
stdW  = sqrt( diag(CovW) );                       %standard errors, White's
CovDKj = zx_1 * Shatj * zx_1';                    %covariance matrix, DK with lags
stdDKj = sqrt(diag(CovDKj) );                    %standard errors, DK with lags


%%%%%%%%%%%%%%%
%%%         %%%
%%% Results %%%
%%%         %%%
%%%%%%%%%%%%%%%

if balanced==1, res.balanced = ' Balanced Sample'; else res.balanced = ' Unbalanced Sample'; end

res.N=N;                                                    % Number of Individuals

if balanced==1

    res.T=T;                                                % Number of Periods (balanced sample)

else

    res.T=zeros(N,1);
    
    for j=1:N
        
        res.T(j)=sum(sample((sample(:,2)==j),1));           % Number of Periods (unbalanced sample)
    end

end

                                           
gam1=parameter(1,1)';                              % Slopes of the r Transition Functions
gam2=parameter(2,1)';                              % Slopes of the r Transition Functions                                                   % Estimated Location Parameters fo each Transition
c=reshape(parameter(3:end,:),1,r);  
beta=reshape(beta,1,K+L+r);
beta_std=real(reshape(sqrt(diag(vcov2)),1,K+L+r));
res.beta=beta;
res.beta_std=stdDKj';
beta_tstat=res.beta./res.beta_std;                       % T-statistics based on Corrected Standard Errors
res.beta_tstat=beta_tstat;
pvalue=2*(1-normcdf(abs(res.beta_tstat)));

res.gam1=gam1; 
res.gam2=gam2; 
res.c=c;



res.vcov2=vcov2;
res.pvalue=pvalue;
res.rss=rss;                                            % Residual Suml of Squares
res.resid=reshape(resid,T,N);                           % Estimated Residuals (Matrix T,N)
res.DW=sum(sum((resi_t-resi_lag1).^2))/SSR;            % The Durbin Watson (DW) statistic 

if nargin<5 

    res.LM_linearity=LM_linearity;                          % Tests of H0 Linearity : Wald test (statistic LMF), Critical Values (10%, 5% and 1%) and Pvalue

    res.F_linearity=F_linearity;                            % Tests of H0 Linearity : Fisher test (statistic LMF), Critical Values (10%, 5% and 1%) and Pvalue

    res.LRT_linearity=LRT_linearity;                        % Tests of H0 Linearity : LRT test, Critical Values (10%, 5% and 1%) and Pvalue

    res.LM=LM;                                              % Tests of Remnaining Non Linearity H0: r H1: r+1: Wald Statistic LM, Critical Values (10%, 5% and 1%) and Pvalue

    res.F=F;                                                % Tests of Remnaining Non Linearity H0: r H1: r+1: Fisher Statistic LMF, Critical Values (10%, 5% and 1%) and Pvalue

    res.LRT=LRT;                                            % Tests of Remnaining Non Linearity H0: r H1: r+1: LRT Statistic, Critical Values (10%, 5% and 1%) and Pvalue

end


res.exitflag=EXITFLAG;                                  % NLS Algorithm Convergence
res.output=OUTPUT;                                      % NLS Algorithm Information
res.nbparam=L+(r+1)+2*1+r;                               % Number of Parameters
loglikelihood=log(res.rss/(N*T-res.nbparam-1));
res.loglikelihood=loglikelihood;
res.AIC=log(res.rss/(N*T-res.nbparam-1))+2*(res.nbparam)/(N*T);
res.BIC=log(res.rss/(N*T-res.nbparam-1))+(res.nbparam)/(N*T)*log(N*T);
res.Rsquared=1-res.rss/TSS;
res.Ad_Rsquared=1-((res.rss/(N*T-res.nbparam))/(TSS/(N*T-1)));
if affi==1
    
    disp(sprintf('  RSS = %4.3f      Convergence = %1.0f   ',rss,EXITFLAG))    

    disp(' '), disp(sprintf('  AIC = %4.3f      BIC = %4.3f   ',res.AIC,res.BIC))
     disp(sprintf('  Rsquared = %4.3f Ad_Rsquared= %4.3f',res.Rsquared, res.Ad_Rsquared))
      disp(sprintf('  DW = %4.3f',res.DW))  
    disp(' '),disp(' Estimated slope parameter of the transition function (one for for each transition function) '),disp(res.gam1)
     disp(' '),disp(' Estimated slope parameter of the transition function (one for for each transition function) '),disp(res.gam2)
    disp(' Estimated location parameters (per column for each transition function) '),disp(res.c)

    disp(' Estimated slope parameters (per column for each transition function) '),disp(res.beta)
    
    disp(' Standard Errors of estimated slope parameters corrected fo heteroskedasticity (per column for each transition function) ')
    disp(res.beta_std)
    
    disp(' t-statistics based on corrected standard errors (per column for each transition function) ')
    disp(res.beta_tstat)
    
    disp(' OUPUT: Individual Elasticities for each explicative variable (first column is the cross section identifier) are available in the excel file result1.xls')
    disp(' ')
    ee=repmat((1:N),length(Q)/N,1);ee=ee(:); 
    resGPSTR=[beta(:) stdDKj(:) beta_tstat(:) pvalue(:)];
    resslope_location=[ c(:); gam1(:); gam2(:)];
    xlswrite('resultGPSTR.xls',resGPSTR);
    xlswrite('resultslop_location.xls',resslope_location);
    xlswrite('resultvcov.xls',CovDKj);
   result1=[];
end
