% PURPOSE: Non Linear Least Squared for Panel Smooth Threshold Models
% -------------------------------------------------------
% Usage: NLS(parameter)  
% 
% -------------------------------------------------------
%  Function : NLS(parameter)
%

function [RSS]=NLS(parameter)

global Y X W Q  T N K Xreg beta residual y_cent x_cent z_cent w_cent g r NT sample Qadj Xadj sampleAdj
gam1=round(parameter(1,1),20);                        % Slope of the Transition Function
gam2=round(parameter(2,1),20);

c=reshape(parameter(3:end,1),1,r);         % Location Parameters
%%%%%%%%%%%%%%%%%%%%%%%%%
%% Transition Function %%
%%%%%%%%%%%%%%%%%%%%%%%%%

g=zeros(NT,r);                                              % Initialisation of Transition Functions
g1=zeros(NT,r);  
g2=zeros(NT,r);  
W=zeros(NT,K,r);                                            % Initialisation of Matrix W

for  i=1:r                                             % Loop on Threshold 

    qsc=repmat(Qadj,1,1)-repmat(c(:,i)',NT,1) ;             % Values of Q(i,t)-c(j); j=1,..,m
if gam1(i,1)<0;
g1(:,i)=1./(1+exp(-(1./gam1(:,i).*log(1-gam1(:,i).*abs(qsc))))) ; 

elseif gam1(i,1)==0;
g1(:,i)=(1./(1+exp(-qsc))) ;

elseif gam1(i,1)>0;
g1(:,i)=1./(exp(-(-1./gam1(:,i).*(exp(gam1(:,i).*abs(qsc))-1)))+1);

end

a1=g1(1:sum(qsc<0),:);

if gam2(i,1)<0;
g2(:,i)=1./(1+exp(-(-1./gam2(:,i).*log(1-gam2(:,i).*abs(qsc))))); 
elseif gam2(i,1)==0;
g2(:,i)=(1./(1+exp(-qsc)));    
elseif gam2(i,1)>0;
g2(:,i)=1./(exp(-(1./gam2(:,i).*(exp(gam2(:,i).*abs(qsc))-1)))+1);

end 

b2=g2(sum(qsc<0)+1:end,:);
g(:,i)= vertcat(a1,b2);   


    W(:,:,i)=Xadj.*repmat(g(:,i),1,K);                      % W(i,t)=X(i,t)*g(qit,c,gam)
    
end

 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% First Step of Estimation : Individual Means % 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Endogeneous Variable and Explicative Variables are computed outside: 
% Only Variables Wj(i,t)=X(i,t)*gj(qit,c,gam) are considered here

Xreg=x_cent;                                                % Matrix of Regressors

for i=1:r                                                   % Loop on the numebr of threshold

    w_cent=zeros(NT,K);                                     % Initialization of Matrix w_cent           
    
    for indic_k=1:K                                         % Loop on the number of explicative variable
        
         w_cent=centered(W(:,indic_k,i),sampleAdj);         % Centered Data        
   
          Xreg=[Xreg w_cent];                               % Matrix of Regressors
          
    end                                                     % End of loop on indic_k  
    
end                                                         % End of Loop on Threshold

%%%%%%%%%%%%%%%%%%%%
% Conditionnal OLS %
%%%%%%%%%%%%%%%%%%%%
Xreg=[Xreg z_cent];  
beta=Xreg\y_cent;                                         % Conditionnal OlS to c and gam

residual=y_cent-Xreg*beta;                                % Vector of Residuals

RSS=sum(residual.^2);                                     % Conditionnal RSS
res.W=W;
%disp(parameter)
%disp(g)
%disp(abs(qsc))